/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var rtls = rtls || {};
var scanResults = [];

const CONN_INT = 80; // 100ms connection interval (n*1.25)

// CC2642R1 AoA related settings
// Receiver params
const aoaSlotDurations = 1; // 1us slots
const aoaSampleRate = 1; // 1Mhz sampling rate
const aoaSampleSize = 1; // 1 byte sample size
const aoaSamplingControl = 0; //
const aoaSamplingEnable = 1; // Enable sampling
const aoaNumAnt = 3; // 3 antennas
const aoaAntArraySwitch = 27; // IO 27 used for array switching
const aoaAntArray = [28, 29, 30]; // IO"s 28, 29, 30 used as antennas

// Transmitter params
const aoaCteLength = 20; // CTE Length is 20 (CTE Time is 20 * 8 = 160 us)
const aoaCteInterval = 1; // Transmit CTE every connection interval


const wsSend = (req) => {
    console.log(">> " + JSON.stringify(req));
    gc.ws.send(JSON.stringify(req));
};

const log = (level, msg) => {
    console.log(msg);
    if (templateObj) {
        templateObj.$.logview.log(level, msg);
    }
};

const controlApi = {
    list_devices: () => wsSend({control: {req: "LIST_DEVICES"}})
};

const syncreq = (identifier, command, payload) => ({identifier, message: {type: "SyncReq", subsystem: "RTLS", command, payload: payload || {}}});

const Device = (devInfo) => {
    const state = {};

    return {
        ...devInfo,
        isMaster: devInfo.caps.indexOf("RTLS_MASTER") > -1,
        identify: () => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_IDENTIFY")),
        reset: () => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_RESET_DEVICE")),
        scan: () => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_SCAN")),
        connect: (addrType, peerAddr, connInterval) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_CONNECT", {addrType, peerAddr, connInterval})),
        terminate: () => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_TERMINATE_LINK")),
        setConnInfo: (accessAddress, connInterval, hopValue, mSCA, currChan, chanMap, crcInit) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_CONN_PARAMS", {accessAddress, connInterval, hopValue, mSCA, currChan, chanMap, crcInit})),
        //
        calibrate: (enable, samplesPerFreq, calibDistance, useCalibFromNV) => wsSend(syncreq(devInfo.identifier, 'RTLS_CMD_TOF_CALIBRATE', { enable, samplesPerFreq, calibDistance, useCalibFromNV})),
        tofStart: (enable) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_TOF_ENABLE", {enable})),
        tofSetParams: (tofRole, numSamples, numFreq, slaveLqiFilter, postProcessLqiThresh, postProcessMagnRatio, autoTofRssiThresh, resultMode, runMode, frequencies) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_TOF_SET_PARAMS", {tofRole, numSamples, numFreq, slaveLqiFilter, postProcessLqiThresh, postProcessMagnRatio, autoTofRssiThresh, resultMode, runMode, frequencies})),
        tofSetSeed: (seed) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_TOF_SET_SEC_SEED", {seed})),
        tofGetSeed: () => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_TOF_GET_SEC_SEED")),
        //
        aoaStart: (enable, cteInterval, cteLength) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_AOA_ENABLE", {enable, cteInterval, cteLength})),
        aoaSetParams: (aoaRole, aoaResultMode, slotDurations, sampleRate, sampleSize, sampleCtrl, samplingEnable, numAnt, antArraySwitch, antArray) => wsSend(syncreq(devInfo.identifier, "RTLS_CMD_AOA_SET_PARAMS", {aoaRole, aoaResultMode, slotDurations, sampleRate, sampleSize, sampleCtrl, samplingEnable, numAnt, antArraySwitch, antArray})),
        //
        state
    };
};

const Request = (msg) => {
    const device = rtls.devices && rtls.devices.find(e => e.identifier === msg.identifier);
    return {
        device: device,
        ...msg.message
    };
};

const handleControl = (control) => {
    switch (control.req) {
        case "LIST_DEVICES": {
            rtls.devices = control.devices.map(e => Device(e));

            rtls.master = rtls.devices.find(e => e.isMaster);
            rtls.passives = rtls.devices.filter(e => !e.isMaster);

            // Error check and mode check
            // 1. Need at least 1 master
            if (!rtls.master) log("error", "Need a master device");

            // 2. All must be either AoA or ToF
            const tofSupported = rtls.devices.every(e => (e.caps.indexOf("TOF_MASTER") >= 0) || (e.caps.indexOf("TOF_PASSIVE") >= 0));
            const aoaSupported = rtls.devices.every(e => (e.caps.indexOf("AOA_RX") >= 0));

            // Bind variables to each device
            rtls.devices.every(e => (e.state.tofSupported = tofSupported));
            rtls.devices.every(e => (e.state.aoaSupported = aoaSupported));

            // Initialize graph
            if (templateObj) {
                const deviceLabels = control.devices.map(e => `${e.port} / ${e.identifier} / ${e.caps.join("+")}`).join("|");
                templateObj.$.lst_connected_devices.labels = deviceLabels;

                var graphNodesToF = [rtls.master].concat(rtls.passives.slice(0, 3));
                var graphNodesAoA = rtls.passives.slice(0, 4);

                var graphAoa = templateObj.$.ti_widget_scalargraph_aoa;
                var graphTof = templateObj.$.ti_widget_scalargraph_tof;

                if (aoaSupported) {
                    graphAoa.titleLabel = "Angle of Arrival";
                    graphAoa.fixedScale = true;
                    graphAoa.numChannels = graphNodesAoA.length;
                    graphNodesAoA.forEach((e, i) => {
                        e.graphIdAoA = i;
                        graphAoa["series_" + i + "_label"] = e.caps.join("+");
                        graphAoa["series_" + i + "_units"] = "degrees";
                        graphAoa["series_" + i + "_min"] = -150;
                        graphAoa["series_" + i + "_max"] = 150;
                    });
                }
                else {
                    graphAoa.titleLabel = "AoA not supported";
                }

                if (tofSupported) {
                    graphTof.titleLabel = "Time of Flight";
                    graphTof.fixedScale = true;
                    graphTof.numChannels = graphNodesToF.length;
                    graphNodesToF.forEach((e, i) => {
                        e.graphIdToF = i;
                        graphTof["series_" + i + "_label"] = e.caps.join("+");
                        graphTof["series_" + i + "_units"] = "meters";
                        graphTof["series_" + i + "_min"] = 0;
                        graphTof["series_" + i + "_max"] = 30;
                    });
                }
                else {
                    graphTof.titleLabel = "ToF not supported";
                }
            }

            // Initialize devices
            scanResults = [];

            rtls.master.terminate();
            rtls.passives.forEach(it => it.reset());
            setTimeout(() => {
                rtls.master.reset();

                // Check if we are to automatically connect
                if (templateObj.$.chk_autortls.checked === true) {
                    setTimeout(() => {
                        rtls.master.scan();
                    }, 2000);
                }
                else {
                    templateObj.$.btn_scan.disabled = false;
                }
            }, 500);

            break;
        }
        default: {
            log("error", "Unknown response: " + control.req);
            break;
        }
    }
};

let peerAddr;
let peerAddrType;
let seed;


const handleRtlsMsg = (req) => {
    //
    // Initialize array for smoothing ToF distance results,
    // each device has its own array and count.
    // Averaging over 4 connection events gives us a reasonable confidence
    // level at +/- 1m confidence interval
    //
    const MAX_RES = 4;
    req.device.state.dist_res = req.device.state.dist_res || [0.0, 0.0, 0.0, 0.0];
    req.device.state.conn_evt_count = req.device.state.conn_evt_count || 0;
    // Static local variables owned by handleRtlsMsg, initial value is false
    handleRtlsMsg.aoaAngleReceived = handleRtlsMsg.aoaAngleReceived || false;
    handleRtlsMsg.tofDistReceived = handleRtlsMsg.tofDistReceived || false;

    //
    // Local Variables related to ToF parameters and calibration
    //
    const freqList = [2408, 2412, 2418, 2424];
    const numFreq = freqList.length;
    const numSyncWords = 256;
    const tofSampleMode = "TOF_MODE_DIST";
    const tofRunMode = "TOF_MODE_CONT";
    const tofRssiThreshold = -55;
    const tofCalibDistance = 1; // One meter
    const tofNumCalibSamples = 1000;
    const slaveLqiFilter    = 255;
    const postProcessLqiThresh = 255;
    const postProcessMagnRatio = 111;

    //
    // Connection establishment to remote device
    //

    // Save latest scan response
    if (req.command === "RTLS_CMD_SCAN" && req.type === "AsyncReq") {
        peerAddr = req.payload.addr;
        peerAddrType = req.payload.addrType;
        gui_scan_result(req);
        update_status_indicators("BLE", "Scan results found", 25);
    }

    // Connect to last found when scan is finished
    if (req.command === "RTLS_CMD_SCAN_STOP") {
        if (templateObj.$.chk_autortls.checked === true) {
            if (peerAddr) {
                rtls.master.connect(peerAddrType, peerAddr, CONN_INT);

                update_status_indicators("BLE", "connecting, pairing, opening L2CAP CoC", 50);
            }
            else {
                update_status_indicators("BLE", "No devices found", 15);
                rtls.master.scan();
            }
        }
        else {
            update_status_indicators("BLE", "Scan completed,\n select the device and connect", 25);
            templateObj.$.btn_conn_disconn.disabled = false;
        }
    }

    // When master sends CONNECT later, pairing is complete and we can initialize AoA and ToF parameters
    if (req.command === "RTLS_CMD_CONNECT" && req.type === "AsyncReq"
        && req.payload.status === "RTLS_SUCCESS") {
        update_status_indicators("BLE", "Connected, paired, L2CAP CoC open", 100);

        if (req.device.state.aoaSupported) {
            const aoaRole = req.device === rtls.master ? "AOA_MASTER" : "AOA_PASSIVE";

            req.device.aoaSetParams(aoaRole, "AOA_MODE_ANGLE", aoaSlotDurations, aoaSampleRate, aoaSampleSize, aoaSamplingControl, aoaSamplingEnable, aoaNumAnt, aoaAntArraySwitch, aoaAntArray);

            update_status_indicators("AOA", "Setting AoA parameters", 50);
        }

        if (req.device.state.tofSupported) {
            const role = req.device === rtls.master ? "TOF_MASTER" : "TOF_PASSIVE";

            req.device.tofSetParams(role , numSyncWords, numFreq, slaveLqiFilter, postProcessLqiThresh, postProcessMagnRatio, tofRssiThreshold, tofSampleMode, tofRunMode, freqList);


            update_status_indicators("TOF", "Setting ToF parameters", 50);

            // Reset object variables
            req.device.state.seedSet = false;
            req.device.state.calibSet = false;
            req.device.state.tofEnabled = false;
            req.device.state.calibDone = false;
            req.device.state.tofSet = false;
        }


        if (templateObj.$.chk_autortls.checked !== true) {
            templateObj.$.btn_enable_rtls.disabled = false;
        }
    }

    //
    // AoA Related
    //

    // After aoa parameters are set, start aoa
    if (req.command === "RTLS_CMD_AOA_SET_PARAMS" && req.payload.status === "RTLS_SUCCESS") {
        if (templateObj.$.chk_autortls.checked === true) {
            update_status_indicators("AOA", "Parameters set, enabling", 80);
            req.device.aoaStart(true, aoaCteInterval, aoaCteLength);
        }
        else {
            update_status_indicators("AOA", "Parameters set, ready to enable", 60);
        }
    }

    //
    // Receive Data
    //

    if (req.command === "RTLS_CMD_AOA_RESULT_ANGLE")
    {
        // Only update status string on first result
        if (handleRtlsMsg.aoaAngleReceived === false) {
            handleRtlsMsg.aoaAngleReceived = true;
            update_status_indicators("AOA", "Angle sampling enabled", 100);
        }

        if (templateObj) {
            const graph = templateObj.$.ti_widget_scalargraph_aoa;
            const id = req.device.graphIdAoA;
            if (id !== undefined) {
                graph["value_" + id.toString()] = req.payload.angle;
            }
        }
    }

    //
    // ToF Related
    //

    // After tof_init, get the random seed from the master
    if (req.command === "RTLS_CMD_TOF_SET_PARAMS") {
        req.device.state.tofSet = true;

        if (rtls.passives.length > 0) {
            // Make sure all nodes have parameters set before moving on
            if (rtls.passives.every(e => e.state.tofSet)
                && rtls.master.state.tofSet === true) {
                rtls.master.tofGetSeed();
                update_status_indicators("TOF", "Parameters set,\n getting seed from master", 60);
            }
        }
        else {
            rtls.master.calibrate(true, tofNumCalibSamples, tofCalibDistance, false);
            update_status_indicators("TOF", "Starting ToF calibration", 70);
        }
    }

    // When we get seed, give it to the passives
    if (req.command === "RTLS_CMD_TOF_GET_SEC_SEED") {
        seed = req.payload.seed;
        rtls.passives.forEach(e => {
            e.tofSetSeed(seed);
        });

        update_status_indicators("TOF", "Sending seed to passives", 65);
    }

    // When all seeds are set, run calibration
    if (req.command === "RTLS_CMD_TOF_SET_SEC_SEED") {
        req.device.state.seedSet = true;

        if (rtls.passives.every(e => e.state.seedSet)) {
            rtls.passives.forEach((e) => {
                e.calibrate(true, tofNumCalibSamples, tofCalibDistance, false);
            });

            rtls.master.calibrate(true, 0, tofCalibDistance, false);
            update_status_indicators("TOF", "Seed set, starting calibration", 70);
        }
    }

    // When calibration is enabled, enable ToF on passives
    if (req.command === "RTLS_CMD_TOF_CALIBRATE" && req.type === "SyncRsp") {
        req.device.state.calibSet = true;

        if (templateObj.$.chk_autortls.checked === true) {
            // Enable ToF on passives after they have calibration enabled
            if (rtls.passives.length > 0) {
                if (rtls.passives.every(e => e.state.calibSet) && req.device !== rtls.master) {
                    rtls.passives.forEach(e => {
                        e.tofStart(1);
                    });
                    update_status_indicators("TOF", "ToF Enabled", 75);
                }

            }
            else {
                rtls.master.tofStart(1);
                update_status_indicators("TOF", "ToF Enabled", 75);
            }
        }
        else {
            templateObj.$.btn_enable_rtls.disabled = false;
        }
    }

    // Once all passives have ToF enabled, start ToF on master
    if (req.command === "RTLS_CMD_TOF_ENABLE" && req.payload.status === "RTLS_SUCCESS") {
        req.device.state.tofEnabled = true;

        // Once all passives have TOF started, start the master.
        if (rtls.passives.every(e => e.state.tofEnabled) && req.device !== rtls.master)
        {
            rtls.master.tofStart(1);
        }
        update_status_indicators("TOF", "ToF Enabled", 75);
    }

    if (req.command === "RTLS_CMD_TOF_CALIBRATE" && req.type === "AsyncReq"
       && rtls.passives.includes(req.device)) {
        req.device.state.calibDone = true;
        if (rtls.passives.every(e => e.state.calibDone)) {
            rtls.master.calibrate(false, 0, tofCalibDistance, false);
        }

        update_status_indicators("TOF", "ToF calibration complete", 95);
    }

    //
    // Receive Data
    //
    if (req.command === "RTLS_CMD_TOF_RESULT_DIST") {
        // Only update status string on first result
        if (handleRtlsMsg.tofDistReceived === false) {
            handleRtlsMsg.tofDistReceived = true;
            update_status_indicators("TOF", "Distance mode enabled", 100);
        }

        // Add the new result to the end and shift out the previous
        req.device.state.dist_res[req.device.state.conn_evt_count] = req.payload.distance;
        req.device.state.conn_evt_count += 1;

        if (req.device.state.conn_evt_count === MAX_RES) {
            // Reset the index
            req.device.state.conn_evt_count = 0;

            // Plot the average of the array
            if (templateObj) {
                const graph = templateObj.$.ti_widget_scalargraph_tof;
                const id = req.device.graphIdToF;
                if (id !== undefined) {
                    // Plot the average of the array at this time
                    graph["value_" + id.toString()] = req.device.state.dist_res.reduce((a, b) => a + b, 0) / req.device.state.dist_res.length;
                }
            }
        }
    }
};

//
// Function to update status Labels
//
const update_status_indicators = (statType, statusString, progressValue) => {
    if (!templateObj) {
        console.log("Can\"t open templateObj! Status update failed");
        return;
    }

    switch (statType) {
        case "BLE":
            templateObj.$.ti_widget_status_label_ble.label = "BLE Status:" + statusString;
            templateObj.$.ti_widget_progressbar_ble.value = progressValue;
            break;
        case "AOA":
            templateObj.$.ti_widget_status_label_aoa.label = "AoA Status:" + statusString;
            templateObj.$.ti_widget_progressbar_aoa.value = progressValue;
            break;
        case "TOF":
            templateObj.$.ti_widget_status_label_tof.label = "ToF Status:" + statusString;
            templateObj.$.ti_widget_progressbar_tof.value = progressValue;
            break;
        default:
          // code block
          break;
      }
};

//
// Function for handling received data from the websocket
//
const ws_receive = (msg) => {
    // console.log(msg.data);
    if (templateObj) templateObj.$.logview.log("info", msg.data);
    try {
        const parsed = JSON.parse(msg.data);

        if (parsed.control) {
            handleControl(parsed.control);
        } else if (rtls.devices) {
            const req = Request(parsed);
            if (req.command === "RTLS_CMD_ERROR" && req.payload.status === "RTLS_ILLEGAL_CMD") log("error", "Illegal command received. Probably tried to do ToF on an AoA node or vice versa");
            // console.log(`${req.command} from ${req.device.name}`);

            handleRtlsMsg(req);
        }
    } catch (err) {
        log("error", err);
    }
};

//
// OnClick handler for btn_scan. Tells master device to scan
// Only used when Auto RTLS is disabled, in case customer wants to choose
// slave based on BD_ADDR
//
const scan_for_slave = () => {
    // Disable the connect button while scanning
    templateObj.$.btn_conn_disconn.disabled = true;

    // Tell the master device to scan
    setTimeout(() => {
        rtls.master.scan();
    }, 500);

};

//
// OnClick handler for btn_conn_disconn. Tells master device to connect to slave
// Only used when Auto RTLS is disabled, in case customer wants to choose
// slave based on BD_ADDR. Takes the actively selected index of lst_remote_devices
//
const connect_disconnect = () => {
    // Get the index of the actively selected slave device
    var idx = templateObj.$.lst_remote_devices.getSelectedIndex();
    if (idx >= 0) {
        // If the index is valid, connect to that device
        rtls.master.connect(scanResults[idx].addrType, scanResults[idx].addr, CONN_INT);
        console.log("Connect to: " + scanResults[idx].addr);

        update_status_indicators("BLE", "Connecting, pairing, opening L2CAP CoC", 50);
    }
    else {
        console.warn("Can't connect, no item selected");
    }

};

//
// OnConnect handler for websocket. Initializes button states and lists devices
//
const wss_connect = () => {
    try {
        var ws_addr = templateObj.$.input_ws_addr.value;
        gc.ws = new WebSocket(ws_addr);
        gc.ws.onmessage = ws_receive;
        gc.ws.onclose = () => {
            templateObj.$.logview.log("error", "Closed");
            templateObj.$.btn_connect.disabled = false;
        };

        gc.ws.onopen = () => {
            controlApi.list_devices();

            // Initialize BLE progress indicators
            templateObj.$.ti_widget_progressbar_ble.visible = true;
            templateObj.$.ti_widget_progressbar_ble.value = 0;
            templateObj.$.ti_widget_status_label_ble.visible = true;


            // Initialize ToF progress indicators
            templateObj.$.ti_widget_progressbar_tof.visible = true;
            templateObj.$.ti_widget_progressbar_tof.value = 0;
            templateObj.$.ti_widget_status_label_tof.visible = true;


            // Initialize AoA progress indicators
            templateObj.$.ti_widget_progressbar_aoa.visible = true;
            templateObj.$.ti_widget_progressbar_aoa.value = 0;
            templateObj.$.ti_widget_status_label_aoa.visible = true;

            // Once we have connected to the websocket, we lock in the settings for Auto RTLS
            templateObj.$.btn_connect.disabled = true;
            templateObj.$.chk_autortls.disabled = true;
        };
    }
    catch (err) {
        templateObj.$.logview.log("error", "Could not connect");
        alert("Could not connect!");
        console.log(err);
    }
};

gc.tofs = gc.tofs || [];

//
// Handler for processing scan results and showing them in lst_remote_devices
//
const gui_scan_result = (req) => {
    if (!templateObj) return;
    const list = templateObj.$.lst_remote_devices;

    scanResults.push({...req.payload});
    list.labels = scanResults.map(e => `${e.addr}`).join("|");
};

const enable_rtls = (req) => {

    if (rtls.devices.every(e => (e.state.tofSupported))) {
        if ( rtls.passives.length > 0) {
            if (rtls.passives.every(e => e.state.calibSet)) {
                rtls.passives.forEach(e => {
                    e.tofStart(1);
                });
                update_status_indicators("TOF", "ToF Enabled", 75);
            }
        }
        else {
            rtls.master.tofStart(1);
            update_status_indicators("TOF", "ToF Enabled", 75);
        }
    }


    if (rtls.devices.every(e => (e.state.aoaSupported))) {
        rtls.passives.forEach(e => {
            e.aoaStart(true, aoaCteInterval, aoaCteLength);
        });
        rtls.master.aoaStart(true, aoaCteInterval, aoaCteLength);

        update_status_indicators("AOA", "Enabling AoA", 80);
    }
};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener("gc-databind-ready", function()
{
    /*
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */

	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind("widget.id.propertyName", "targetVariable == 1 ? "binding is one" : "binding is not one"");

	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind("widget.id.propertyName", "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

	// Event 1 to n bindings
	/*
    gc.databind.registry.bind("widget.date.value",
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: "widget.dayOfWeek.selectedText",
            day: "widget.dayOfMonth.value",
            month: "widget.month.selectedText",
            year: "widget.year.value"
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id "date"
            return values.weekday + ", " + values.month + " " + values.day + ", " + values.year;
        }
    );
	*/
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener("gc-nav-ready", function()
{
    /*
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */

	// For example,
	// gc.nav.registerAction("myCustomCloseAction", function() { window.close(); }, function() { return true; }, function() { return true; });

	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction("myCustomCloseAction);    then enable it again using:  gc.nav.enableAction("myCustomCloseAction");
});


/*
*  Boilerplate code for working with components in the application gist
*/


var initComplete = false;
var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector("#template_obj");

    // Wait for the template to fire a dom-change event to indicate that it has been "stamped"
    // before trying to access components in the application.
	templateObj.addEventListener("dom-change", function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use "automatic node finding" $ syntax to access widgets.
  	        // e.g. to access a widget with an id of "widget_id" you can use templateObj.$.widgetId

  	        templateObj.$.btn_connect.onclick = wss_connect;
            templateObj.$.btn_connect.disabled = false;
            templateObj.$.chk_autortls.checked = true;
            templateObj.$.chk_autortls.disabled = false;

            templateObj.$.btn_scan.onclick = scan_for_slave;
            templateObj.$.btn_scan.disabled = true;
            templateObj.$.btn_conn_disconn.disabled = true;
            templateObj.$.btn_conn_disconn.onclick = connect_disconnect;
            templateObj.$.btn_enable_rtls.disabled = true;
            templateObj.$.btn_enable_rtls.onclick = enable_rtls;

            // Initialize BLE progress indicators
            templateObj.$.ti_widget_progressbar_ble.visible = false;
            templateObj.$.ti_widget_progressbar_ble.value = 0;
            templateObj.$.ti_widget_status_label_ble.visible = false;


            // Initialize ToF progress indicators
            templateObj.$.ti_widget_progressbar_tof.visible = false;
            templateObj.$.ti_widget_progressbar_tof.value = 0;
            templateObj.$.ti_widget_status_label_tof.visible = false;


            // Initialize AoA progress indicators
            templateObj.$.ti_widget_progressbar_aoa.visible = false;
            templateObj.$.ti_widget_progressbar_aoa.value = 0;
            templateObj.$.ti_widget_status_label_aoa.visible = false;

            templateObj.$.btn_reload.onclick = () => document.location.reload();

        },1);

	});
};

templateObj = document.querySelector("#template_obj");
if (templateObj) {
    init();
} else {
    document.addEventListener("DOMContentLoaded", init.bind(this));
}

